import asyncio
import gzip
import json
import sqlite3
import time
import os
from io import BytesIO
from typing import Callable, Any, List

from hdfs import InsecureClient


class HDFSDataProcessor:
    def __init__(
            self,
            hdfs_url="http://VIP-DC-MASTER-2:9870",
            hdfs_user="root",
            db_file="processed_files.db",
            batch_size=50,
            retry_limit=3,
    ):
        self.hdfs_url = hdfs_url
        self.hdfs_user = hdfs_user
        self.db_file = db_file
        self.batch_size = batch_size
        self.retry_limit = retry_limit
        self.client = InsecureClient(self.hdfs_url, user=self.hdfs_user)
        self.read_hdfs_fanc = {"all": self.all_read_gz, "batch": self.batch_read_gz}
        self.read_hdfs_model = "all"
        self.init_db()

    def init_db(self):
        """初始化 SQLite 数据库"""
        with sqlite3.connect(self.db_file) as conn:
            cursor = conn.cursor()
            cursor.execute("""
                    CREATE TABLE IF NOT EXISTS processed_files (
                        file_path TEXT PRIMARY KEY
                    )
                    """)
            conn.commit()

    def save_processed_file(self, file_path):
        """保存处理过的文件"""
        with sqlite3.connect(self.db_file) as conn:
            cursor = conn.cursor()
            cursor.execute("INSERT OR IGNORE INTO processed_files (file_path) VALUES (?)", (file_path,))
            conn.commit()

    def is_file_processed(self, file_path):
        """检查文件是否已处理"""
        with sqlite3.connect(self.db_file) as conn:
            cursor = conn.cursor()
            cursor.execute("SELECT file_path FROM processed_files WHERE file_path = ?", (file_path,))
            result = cursor.fetchone()
        return result is not None

    def list_gz_files(self, hdfs_dir):
        """列出 HDFS 目录中的所有 gzip 文件"""
        return [f"{hdfs_dir}/{file[0]}" for file in self.client.list(hdfs_dir, status=True) if file[0].endswith(".gz")]

    def count_total_lines(self, gz_file_path: str):
        with self.client.read(gz_file_path) as hdfs_file:
            with gzip.GzipFile(fileobj=hdfs_file) as gz:
                return sum(1 for _ in gz)

    def batch_read_gz(self, gz_file_path: str):
        """分批读取 gz 文件"""
        with self.client.read(gz_file_path) as hdfs_file:
            with gzip.GzipFile(fileobj=hdfs_file) as gz:
                while True:
                    lines = []
                    for _ in range(self.batch_size):
                        try:
                            line = next(gz)
                            if line.strip():  # 移除空行
                                lines.append(line.decode("utf-8"))  # 解码
                        except StopIteration:  # 文件已读完
                            break
                    if not lines:
                        break
                    yield lines

    def all_read_gz(self, gz_file_path: str, encoding='utf-8'):
        """
        读取 HDFS 上的 .gz 文件内容。
        :param hdfs_path: HDFS 文件路径（必须以 .gz 结尾）
        :param encoding: 文件编码格式（默认 utf-8）
        :return: 文件内容
        """
        with self.client.read(gz_file_path) as reader:  # 以二进制模式读取
            compressed_data = reader.read()  # 读取压缩数据
            with gzip.GzipFile(fileobj=BytesIO(compressed_data)) as gz_file:  # 解压缩
                content = gz_file.read().decode(encoding)  # 解码为字符串
                print(f"文件读取成功: {gz_file_path}")
                lines = [i for i in content.splitlines() if i.strip()]
                result = [lines[i:i + self.batch_size] for i in range(0, len(lines), self.batch_size)]
                return result

    async def process_data(self, data, process_func):
        """处理数据并执行处理函数"""
        retry_count = 0
        while retry_count < self.retry_limit:
            try:
                await process_func(data)
                return  # 成功处理后退出
            except Exception as e:
                retry_count += 1
                print(f"处理数据时发生错误: {e}, 正在重试 {retry_count}/{self.retry_limit}, data: {data}")
                await asyncio.sleep(2 ** retry_count)
        print(f"处理数据失败, 达到重试上限, data: {data}")

    async def process_file(self, hdfs_file_path, process_func):
        """处理单个 gz 文件"""
        total_lines = self.count_total_lines(hdfs_file_path)
        processed_lines = 0
        start_time = time.time()
        #   # 这里根据不同的配置选用不同的读取文件的方法
        for lines in self.read_hdfs_fanc[self.read_hdfs_model](hdfs_file_path):
            processing_start_time = time.time()  # 记录本批处理开始时间

            tasks = []
            for line in lines:
                try:
                    data = json.loads(line)
                    tasks.append(self.process_data(data, process_func))
                except json.JSONDecodeError as e:
                    print(f"解析JSON失败: {e}, 行内容: {line.strip()}")

            # await AsyncTaskPool(self.batch_size).run(tasks) # AsyncTaskPool 适用于一次提交所有任务, 限制并发数执行
            await asyncio.gather(*tasks)

            processed_lines += len(lines)

            elapsed_time = time.time() - start_time  # 已用时间
            processing_time = time.time() - processing_start_time  # 本次处理时间
            avg_processing_time = (
                (elapsed_time * 1000) / processed_lines if processed_lines > 0 else float("inf")
            )  # 平均每条数据的处理时间（毫秒）

            # 估算剩余时间
            remaining_time = (
                ((avg_processing_time / 1000) * (total_lines - processed_lines))
                if processed_lines > 0
                else float("inf")
            )

            # 显示总进度信息
            print(
                f"文件: {hdfs_file_path} 总进度: {processed_lines}/{total_lines} 行 | "
                f"已用时间: {elapsed_time:.2f}秒 | 本次处理时间: {processing_time:.2f}秒 | "
                f"预估剩余时间: {remaining_time:.2f}秒 | 平均每条处理时间: {avg_processing_time:.2f}毫秒"
            )

        # 最终进度显示
        final_elapsed_time = time.time() - start_time  # 最终已用时间
        print(
            f"文件: {hdfs_file_path} 处理完成 | 总进度: {processed_lines}/{total_lines} 行 | "
            f"总已用时间: {final_elapsed_time:.2f}秒 | "
            f"平均每条处理时间: {(final_elapsed_time * 1000) / processed_lines:.2f}毫秒"
            if processed_lines > 0
            else "处理无数据"
        )

        self.save_processed_file(hdfs_file_path)  # 保存处理过的文件

    async def retry_process_file(self, hdfs_file_path, process_func):
        """带重试机制的文件处理"""
        retry_count = 0
        while retry_count < self.retry_limit:
            try:
                await self.process_file(hdfs_file_path, process_func)
                return True  # 成功处理后退出
            except Exception as e:
                retry_count += 1
                print(f"处理文件 {hdfs_file_path} 时发生错误: {e}，正在重试 {retry_count}/{self.retry_limit}")
                await asyncio.sleep(2 ** retry_count)
        print(f"处理文件 {hdfs_file_path} 失败，达到重试上限")
        return False
        # raise

    async def batch_process_file(self, hdfs_dir: str, process_func: Callable[[dict], Any]):
        """批量更新所有 gz 文件"""
        gz_files = self.list_gz_files(hdfs_dir)
        all_succeed = True
        for hdfs_file_path in gz_files:
            if self.is_file_processed(hdfs_file_path):
                print(f"跳过已处理文件: {hdfs_file_path}")
                continue  # 如果文件已处理，跳过
            succeed = await self.retry_process_file(hdfs_file_path, process_func)  # 处理文件
            if succeed is False:
                all_succeed = False

        if all_succeed:
            # 处理完成后删除数据库文件
            try:
                if os.path.exists(self.db_file):
                    os.remove(self.db_file)
                    print(f"已删除断点重试文件: {self.db_file}")
            except Exception as e:
                print(f"删除断点重试文件失败: {e}")

    async def process_file_bulk(self, hdfs_file_path, process_func):
        """按批次处理单个文件，批量数据传递给处理函数"""
        total_lines = self.count_total_lines(hdfs_file_path)
        processed_lines = 0
        start_time = time.time()

        tasks = []
        # 这里根据不同的配置选用不同的读取文件的方法
        for lines in self.read_hdfs_fanc[self.read_hdfs_model](hdfs_file_path):
            processing_start_time = time.time()  # 记录本批处理开始时间

            batch_data = []
            for line in lines:
                try:
                    data = json.loads(line)
                    batch_data.append(data)
                except json.JSONDecodeError as e:
                    print(f"解析JSON失败: {e}, 行内容: {line.strip()}")

            # 处理读取到的批次数据
            if batch_data:
                tasks.append(process_func(batch_data))  # 将批次数据传递给处理函数并收集任务
                processed_lines += len(batch_data)  # 更新已处理行数

            # 当积累的任务数量达到 batch_size 时并发处理所有任务
            if len(tasks) >= self.batch_size:
                await asyncio.gather(*tasks)  # 同时处理多个批次

                elapsed_time = time.time() - start_time  # 已用时间
                processing_time = time.time() - processing_start_time  # 本次处理时间
                avg_processing_time = (
                    (elapsed_time * 1000) / processed_lines if processed_lines > 0 else float("inf")
                )  # 平均每条数据的处理时间（毫秒）

                # 估算剩余时间
                remaining_time = (
                    ((avg_processing_time / 1000) * (total_lines - processed_lines))
                    if processed_lines > 0
                    else float("inf")
                )

                # 显示总进度信息
                print(
                    f"文件: {hdfs_file_path} 总进度: {processed_lines}/{total_lines} 行 | "
                    f"已用时间: {elapsed_time:.2f}秒 | 本次处理时间: {processing_time:.2f}秒 | "
                    f"预估剩余时间: {remaining_time:.2f}秒 | 平均每条处理时间: {avg_processing_time:.2f}毫秒"
                )

                # 清空任务列表，准备下一批处理
                tasks.clear()
            # 处理剩余的任务
        if tasks:
            await asyncio.gather(*tasks)  # 处理未达到 batch_size 的剩余任务

        # 最终进度显示
        final_elapsed_time = time.time() - start_time  # 最终已用时间
        print(
            f"文件: {hdfs_file_path} 处理完成 | 总进度: {processed_lines}/{total_lines} 行 | "
            f"总已用时间: {final_elapsed_time:.2f}秒 | "
            f"平均每条处理时间: {(final_elapsed_time * 1000) / processed_lines:.2f}毫秒"
            if processed_lines > 0
            else "处理无数据"
        )

        self.save_processed_file(hdfs_file_path)

    async def retry_process_file_bulk(self, hdfs_file_path, process_func):
        """带重试机制的批量文件处理"""
        retry_count = 0
        while retry_count < self.retry_limit:
            try:
                await self.process_file_bulk(hdfs_file_path, process_func)
                return True  # 成功处理后退出
            except Exception as e:
                retry_count += 1
                print(f"处理文件 {hdfs_file_path} 时发生错误: {e}，正在重试 {retry_count}/{self.retry_limit}")
                await asyncio.sleep(2 ** retry_count)
        print(f"处理文件 {hdfs_file_path} 失败，达到重试上限")
        return False

    async def batch_process_file_bulk(self, hdfs_dir: str, process_func: Callable[[List[dict]], Any]):
        """批量处理 gz 文件中的数据"""
        gz_files = self.list_gz_files(hdfs_dir)
        all_succeed = True
        for hdfs_file_path in gz_files:
            if self.is_file_processed(hdfs_file_path):
                print(f"跳过已处理文件: {hdfs_file_path}")
                continue  # 跳过已处理文件
            succeed = await self.retry_process_file_bulk(hdfs_file_path, process_func)
            if succeed is False:
                all_succeed = False

        if all_succeed:
            # 处理完成后删除数据库文件
            try:
                if os.path.exists(self.db_file):
                    os.remove(self.db_file)
                    print(f"已删除断点重试文件: {self.db_file}")
            except Exception as e:
                print(f"删除断点重试文件失败: {e}")

# # 使用示例
# async def update_refer(data: dict):
#     ref_id = data["ref_id"]
#     url = f"http://192.168.98.79:8150/v1/fact_refer/update/{ref_id}"
#     update_data = data["update_data"]
#     if not update_data:
#         return
#
#     # 此处为实际处理逻辑
#     await ApiNetUtils.fetch_post(url=url, payload=update_data)
#
#
# if __name__ == "__main__":
#     processor = HDFSDataProcessor()  # 实例化数据处理类
#     asyncio.run(processor.batch_process_file("/user/libaiyun/output/confidence", update_refer))



import base64
import json


class BytesEncoder(json.JSONEncoder):
    def default(self, obj):
        if isinstance(obj, bytes):
            return base64.b64encode(obj).decode('utf-8')
        return super().default(obj)

# json.dumps(x, ensure_ascii=False, cls=BytesEncoder)


import re

import unicodedata


def is_all_english_chars(s):
    return bool(re.match(r'^[A-Za-z]+$', s))


def contains_chinese_chars(s):
    return bool(re.search(r'[\u3400-\u9fff]', s))


def is_empty(value):
    """
    判断一个值是否为空。

    支持的类型：
    - None
    - 空字符串（去除空白后）
    - pandas 的 NaN
    - 其他可迭代类型（如列表、字典等）的长度为 0
    - 其他情况返回 False
    """
    # 如果是 None，直接返回 True
    if value is None:
        return True

    # 尝试处理 pandas 的 NaN
    try:
        import pandas as pd
        if pd.isna(value):
            return True
    except:
        pass  # 如果没有安装 pandas，跳过

    # 如果是字符串，检查去除空白后是否为空
    if isinstance(value, str):
        return value.strip() == ""

    # 处理其他可迭代类型（如列表、字典等）
    if hasattr(value, "__len__"):
        return len(value) == 0

    # 默认情况下，非 None、非空类型返回 False
    return False


class InvalidCharLengthError(Exception):
    """自定义异常类，用于处理输入字符长度不为 1 的情况"""
    pass


def is_single_cjk_char(char):
    """
    判断单个字符是否为中日韩字符
    :param char: 要判断的单个字符
    :return: 如果是中日韩字符返回 True，否则返回 False
    """
    # 检查输入字符的长度
    if len(char) != 1:
        raise InvalidCharLengthError("输入的字符串长度必须为 1，请提供单个字符进行判断。")
    code_point = ord(char)
    # 中日韩统一表意文字
    ranges = [
        (0x4E00, 0x9FFF),  # CJK 统一表意符号
        (0x3400, 0x4DBF),  # CJK 统一表意符号扩展 A
        (0x20000, 0x2A6DF),  # CJK 统一表意符号扩展 B
        (0x2A700, 0x2B73F),  # CJK 统一表意符号扩展 C
        (0x2B740, 0x2B81F),  # CJK 统一表意符号扩展 D
        (0x2B820, 0x2CEAF),  # CJK 统一表意符号扩展 E
        (0x2CEB0, 0x2EBEF),  # CJK 统一表意符号扩展 F
        (0x30000, 0x3134F),  # CJK 统一表意符号扩展 G
        (0x31350, 0x323AF),  # CJK 统一表意符号扩展 H
        (0x3300, 0x33FF),  # CJK 兼容符号
        (0xFE30, 0xFE4F),  # CJK 兼容形式
        (0xF900, 0xFAFF),  # CJK 兼容表意符号
        (0x2F800, 0x2FA1F),  # CJK 兼容表意符号补充
        (0x3105, 0x3129),  # 注音字母
        (0x31A0, 0x31BF),  # 注音字母扩展
        (0x3040, 0x309F),  # 平假名
        (0x30A0, 0x30FF),  # 片假名
        (0x31F0, 0x31FF),  # 片假名扩展
        (0xAC00, 0xD7AF),  # 韩文音节
        (0x1100, 0x11FF),  # 韩文字母
        (0xA960, 0xA97F),  # 韩文字母扩展 A
        (0xD7B0, 0xD7FF),  # 韩文字母扩展 B
    ]
    for start, end in ranges:
        if start <= code_point <= end:
            return True
    return False


def is_all_symbols(text):
    # 是否全是符号
    # 如果字符串为空，返回 False
    if not text:
        return False

    # 检查每个字符是否属于符号类别
    return all(unicodedata.category(char).startswith(('P', 'S')) for char in text)